/* ***********************************
  * $ grunt tasks:
  * 
  * $ grunt releaseme           - 'stage', 'release'
  * $ grunt stage             - git add files before running the release task
  * $ grunt updateSubmodules  - updates submodules (ie - angular-fullstack-deps)
  * $ grunt generateDemo      - generate demo app
  * $ grunt releaseDemoBuild  - builds and releases demo
  * $ grunt updateFixtures    - updates package and bower fixtures
  * $ grunt test              - gee, i wonder.
  * $ grunt deps              - update dependencies
  * $ grunt demo              - clean and run generateDemo
  * $ grunt releaseDemo       - release the kraken!
  *
  * ***********************************
*/
  
'use strict';

var shell = require('shelljs');
var child_process = require('child_process');
var Q = require('q');
var helpers = require('yeoman-generator').test;
var fs = require('fs');
var path = require('path');

module.exports = function (grunt) {
  var gruntUtils = require('./task-utils/grunt')(grunt);
  var gitCmd = gruntUtils.gitCmd;
  var gitCmdAsync = gruntUtils.gitCmdAsync;

  // Load grunt tasks automatically, when needed
  require('jit-grunt')(grunt, {
    buildcontrol: 'grunt-build-control'
  });

  grunt.initConfig({
    config: {
      demo: 'demo'
    },
    pkg: grunt.file.readJSON('package.json'),
    conventionalChangelog: {
      options: {
        changelogOpts: {
          // conventional-changelog options go here
          preset: 'angular'
        },
        writerOpts: {
          // conventional-changelog-writer options go here
          finalizeContext: gruntUtils.conventionalChangelog.finalizeContext,
          commitPartial: gruntUtils.conventionalChangelog.commitPartial
        }
      },
      release: {
        src: 'CHANGELOG.md'
      }
    },
    release: {
      options: {
        commitMessage: '<%= version %>',
        tagName: '<%= version %>',
        file: 'package.json',
        beforeBump: ['updateSubmodules'],
        afterBump: ['updateFixtures:deps', 'commitNgFullstackDeps'],
        beforeRelease: ['stage'],
        push: true,
        pushTags: true,
        npm: false
      }
    },
    updateSubmodules: {
      options: {
        modules: ['angular-fullstack-deps']
      }
    },
    commitNgFullstackDeps: {
      options: {
        cwd: 'angular-fullstack-deps',
        files: ['package.json', 'bower.json']
      }
    },
    stage: {
      options: {
        files: ['CHANGELOG.md', 'angular-fullstack-deps']
      }
    },
    buildcontrol: {
      options: {
        dir: 'demo',
        commit: true,
        push: true,
        connectCommits: false,
        message: 'Built using Makeme - Fullstack Anglar/Express/Node/Mongo/Socket v<%= pkg.version %> from commit %sourceCommit%'
      },
      release: {
        options: {
          remote: 'origin',
          branch: 'master'
        }
      }
    },
    jshint: {
      options: {
        curly: false,
        node: true
      },
      all: ['Gruntfile.js', '*/index.js']
    },
    env: {
      fast: {
        SKIP_E2E: true
      }
    },
    mochaTest: {
      test: {
        src: [
          'test/*.js'
        ],
        options: {
          reporter: 'spec',
          timeout: 120000
        }
      }
    },
    clean: {
      demo: {
        files: [{
          dot: true,
          src: [
            '<%= config.demo %>/*',
            '!<%= config.demo %>/readme.md',
            '!<%= config.demo %>/node_modules',
            '!<%= config.demo %>/.git',
            '!<%= config.demo %>/dist'
          ]
        }]
      }
    },
    fixtures: {
      gen: {
        options: {}
      },
      app: {
        options: {
          package: 'test/fixtures/package.json'
        }
      }
    }
  });

  grunt.registerTask('stage', 'git add files before running the release task', function () {
    var files = grunt.config('stage.options').files;
    gitCmd(['add'].concat(files), {}, this.async());
  });

  grunt.registerTask('updateSubmodules', function() {
    grunt.config.requires('updateSubmodules.options.modules');
    var modules = grunt.config.get('updateSubmodules').options.modules;

    Q()
      .then(gitCmdAsync(['submodule', 'update', '--init', '--recursive']))
      .then(function() {
        var thens = [];
        for (var i = 0, modulesLength = modules.length; i < modulesLength; i++) {
          var opts = {cwd: modules[i]};
          thens.push(gitCmdAsync(['checkout', 'master'], opts));
          thens.push(gitCmdAsync(['fetch'], opts));
          thens.push(gitCmdAsync(['pull'], opts));
        }
        return thens.reduce(Q.when, Q());
      })
      .catch(grunt.fail.fatal.bind(grunt.fail))
      .finally(this.async());
  });

  grunt.registerTask('commitNgFullstackDeps', function() {
    grunt.config.requires(
      'commitNgFullstackDeps.options.files',
      'commitNgFullstackDeps.options.cwd'
    );
    var ops = grunt.config.get('commitNgFullstackDeps').options;
    var version = require('./package.json').version || 'NO VERSION SET';
    if (Array.isArray(ops.files) && ops.files.length > 0) {
      gitCmd(['commit', '-m', version].concat(ops.files), {
        cwd: path.resolve(__dirname, ops.cwd)
      }, this.async());
    } else {
      grunt.log.writeln('No files were commited');
    }
  });

  grunt.registerTask('generateDemo', 'generate demo', function () {
    var done = this.async();

    shell.mkdir(grunt.config('config').demo);
    shell.cd(grunt.config('config').demo);

    Q()
      .then(generateDemo)
      .then(function() {
        shell.cd('../');
      })
      .catch(function(msg){
        grunt.fail.warn(msg || 'failed to generate demo')
      })
      .finally(done);

    function generateDemo() {
      var deferred = Q.defer();
      var options = {
        script: 'js',
        markup: 'html',
        stylesheet: 'sass',
        router: 'uirouter',
        material: true,
        bootstrap: false,
        uibootstrap: false,
        mongoose: true,
        testing: 'jasmine',
        auth: true,
        oauth: ['googleAuth', 'twitterAuth'],
        socketio: true
      };

      var deps = [
        '../app',
        [
          helpers.createDummyGenerator(),
          'ng-component:app'
        ]
      ];

      var gen = helpers.createGenerator('makeme:app', deps);

      helpers.mockPrompt(gen, options);
      gen.run({}, function () {
        deferred.resolve();
      });

      return deferred.promise;
    }
  });

  
  grunt.registerTask('releaseDemoBuild', 'builds and releases demo', function () {
    var done = this.async();

    shell.cd(grunt.config('config').demo);

    Q()
      .then(gruntBuild)
      .then(gruntRelease)
      .then(function() {
        shell.cd('../');
      })
      .catch(function(msg){
        grunt.fail.warn(msg || 'failed to release demo')
      })
      .finally(done);

    function run(cmd) {
      var deferred = Q.defer();
      var generator = shell.exec(cmd, {async:true});
      generator.stdout.on('data', function (data) {
        grunt.verbose.writeln(data);
      });
      generator.on('exit', function (code) {
        deferred.resolve();
      });

      return deferred.promise;
    }

    function gruntBuild() {
      return run('grunt');
    }

    function gruntRelease() {
      return run('grunt buildcontrol:heroku');
    }
    
  });

  grunt.registerTask('updateFixtures', 'updates package and bower fixtures', function(target) {
    var genVer = require('./package.json').version;
    var dest = __dirname + ((target === 'deps') ? '/angular-fullstack-deps/' : '/test/fixtures/');
    var appName = (target === 'deps') ? 'angular-fullstack-deps' : 'tempApp';

    var processJson = function(s, d) {
      // read file, strip all ejs conditionals, and parse as json
      var json = JSON.parse(fs.readFileSync(path.resolve(s), 'utf8').replace(/<%(.*)%>/g, ''));
      // set properties
      json.name = appName, json.version = genVer;
      if (target === 'deps') { json.private = false; }
      // stringify json and write it to the destination
      fs.writeFileSync(path.resolve(d), JSON.stringify(json, null, 2));
    };

    processJson('app/templates/_package.json', dest + 'package.json');
    processJson('app/templates/_bower.json', dest + 'bower.json');
  });

  grunt.registerTask('installFixtures', 'install package and bower fixtures', function() {
    var done = this.async();

    shell.cd('test/fixtures');
    grunt.log.ok('installing npm dependencies for generated app');
    child_process.exec('npm install --quiet', {cwd: '../fixtures'}, function (error, stdout, stderr) {

      grunt.log.ok('installing bower dependencies for generated app');
      child_process.exec('bower install', {cwd: '../fixtures'}, function (error, stdout, stderr) {

        if(!process.env.SAUCE_USERNAME) {
          grunt.log.ok('running npm run-script update-webdriver');
          child_process.exec('npm run-script update-webdriver', function() {
            shell.cd('../../');
            done();
          });
        } else {
          shell.cd('../../');
          done();
        }
      })
    });
  });

  grunt.registerTask('test', function(target, option) {
    if (target === 'fast') {
      grunt.task.run([
        'env:fast'
      ]);
    }

    return grunt.task.run([
      'updateFixtures',
      'installFixtures',
      'mochaTest'
    ])
  });

  grunt.registerTask('deps', function(target) {
    if (!target || target === 'app') grunt.task.run(['updateFixtures']);
    grunt.task.run(['fixtures:' + (target || '')]);
  });

  grunt.registerTask('demo', [
    'clean:demo',
    'generateDemo'
  ]);

  grunt.registerTask('releaseDemo', [
    'demo',
    'releaseDemoBuild',
    'buildcontrol:release'
  ]);

  grunt.registerTask('releaseme', ['stage', 'release']);
};
